@@ -41,12 +41,16 @@
  *              - support for Unix98 ptys with linux-2.2.x and glibc-2.1
  *----------------------------------------------------------------------*/
 
+
 #ifndef lint
-static const char rcsid[] = "$Id: command.c,v 1.7 2001/09/06 15:57:04 sashav Exp $";
+static const char rcsid[] = "$Id: command.c,v 1.19 2005/06/21 20:08:16 sasha Exp $";
 #endif
 
 /*{{{ includes: */
 #include "rxvt.h"		/* NECESSARY */
+#ifdef HAVE_AFTERSTEP
+#include "libAfterStep/event.h"
+#endif
 
 #define TT_PRINTF_LIMIT		1024
 
@@ -95,6 +99,67 @@ static char    *v_bufstr = NULL;	/* beginning of area to write */
 static char    *v_bufptr;	/* end of area to write */
 static char    *v_bufend;	/* end of physical buffer */
 
+#define UPDATE_BACKGROUND_TIMEOUT_SEC	1
+#define UPDATE_BACKGROUND_TIMEOUT_USEC	350000  /* third of a second delay */  
+static time_t 		last_update_background_request_sec = 0 ;  	
+static time_t 		last_update_background_request_usec = 0 ;  	 
+static Bool first_background_update = True ;	 
+
+
+static void
+timer_get_time (time_t * sec, time_t * usec)
+{
+	struct timeval tv;
+
+	gettimeofday (&tv, NULL);
+	*sec = tv.tv_sec;
+	*usec = tv.tv_usec;
+}
+
+
+/* PROTO */
+void 
+request_background_update()
+{
+	Bool repetitive = (last_update_background_request_sec!=0);
+	timer_get_time (&last_update_background_request_sec, &last_update_background_request_usec );
+	if( first_background_update ) 
+	{	
+		last_update_background_request_usec += 5000;
+	}else
+		last_update_background_request_usec += repetitive?UPDATE_BACKGROUND_TIMEOUT_USEC:UPDATE_BACKGROUND_TIMEOUT_USEC*2 ;
+	while( last_update_background_request_usec > 1000000 )
+	{
+		++last_update_background_request_sec ; 
+		last_update_background_request_usec -= 1000000 ;
+	}	 
+#ifdef DEBUG_BACKGROUND_PMAP
+	fprintf( stderr,  "!!! %s\n", __FUNCTION__);
+#endif
+}	 
+
+/* PROTO */
+void 
+cancel_background_update()
+{
+	last_update_background_request_sec = 0;
+#ifdef DEBUG_BACKGROUND_PMAP
+	fprintf( stderr,  "!!! %s\n", __FUNCTION__);
+#endif
+}	 
+
+/* PROTO */
+void 
+set_background_updated()
+{
+	first_background_update = False ;		   
+	last_update_background_request_sec = 0;
+#ifdef DEBUG_BACKGROUND_PMAP
+	fprintf( stderr,  "!!! %s\n", __FUNCTION__);
+#endif
+}	 
+
+
 void get_ourmods(void);
 int pixmap_error_handler (Display * dpy, XErrorEvent * error);
 
@@ -492,6 +557,7 @@ get_pty(void)
 	    grantpt(fd);	/* change slave permissions */
 	    unlockpt(fd);	/* unlock slave */
 	    ptydev = ttydev = ptsname(fd);	/* get slave's name */
+	    goto Found;
 	}
     }
 #elif defined (_AIX)
@@ -604,27 +670,29 @@ get_tty(void)
     ioctl(fd, I_PUSH, "ttcompat");
 #else				/* __svr4__ */
     {
-    /* change ownership of tty to real uid and real group */
-	unsigned int    mode = 0622;
-	gid_t           gid = getgid();
+    	/* change ownership of tty to real uid and real group */
+#ifndef __CYGWIN32__		
+		unsigned int    mode = 0622;
+		gid_t           gid = getgid();
 
 # ifdef TTY_GID_SUPPORT
-	{
-	    struct group   *gr = getgrnam("tty");
+		{
+	    	struct group   *gr = getgrnam("tty");
 
-	    if (gr) {
-	    /* change ownership of tty to real uid, "tty" gid */
-		gid = gr->gr_gid;
-		mode = 0620;
-	    }
-	}
+	    	if (gr) {
+	    	/* change ownership of tty to real uid, "tty" gid */
+			gid = gr->gr_gid;
+			mode = 0620;
+	    	}
+		}
 # endif				/* TTY_GID_SUPPORT */
-	privileges(RESTORE);
-# ifndef __CYGWIN32__
-	fchown(fd, getuid(), gid);	/* fail silently */
-	fchmod(fd, mode);
-# endif
-	privileges(IGNORE);
+#endif
+		privileges(RESTORE);
+#ifndef __CYGWIN32__
+		fchown(fd, getuid(), gid);	/* fail silently */
+		fchmod(fd, mode);
+#endif
+		privileges(IGNORE);
     }
 #endif				/* __svr4__ */
 
@@ -938,7 +1006,7 @@ run_command(char *argv[])
     on_exit(clean_exit, NULL);	/* non-ANSI exit handler */
 # else
 #  ifdef UTMP_SUPPORT
-    print_error("no atexit(), UTMP entries can't be cleaned");
+    print_error("no atexit(), UTMP entries may not be cleaned");
 #  endif
 # endif
 #endif
@@ -975,6 +1043,8 @@ run_command(char *argv[])
     signal(SIGTERM, Exit_signal);
     signal(SIGCHLD, Child_signal);
 
+	tt_winsize(ptyfd);		/* set window size */
+
 /* need to trap SIGURG for SVR4 (Unixware) rlogin */
 /* signal (SIGURG, SIG_DFL); */
 
@@ -1014,7 +1084,6 @@ run_command(char *argv[])
 	    }
 #endif				/* SRIOCSREDIR */
 	}
-	tt_winsize(0);		/* set window size */
 
     /* reset signals and spin off the command interpreter */
 	signal(SIGINT, SIG_DFL);
@@ -1030,6 +1099,12 @@ run_command(char *argv[])
 	signal(SIGTTOU, SIG_IGN);
 #endif				/* SIGTSTP */
 
+#ifdef __CYGWIN__
+    /* cygwin does not close files properly */
+    if( Xfd > 0 )
+        close(Xfd);
+#endif
+
     /* command interpreter path */
 	if (argv != NULL) {
 #ifdef DEBUG_CMD
@@ -1044,7 +1119,7 @@ run_command(char *argv[])
 	    const char     *argv0, *shell;
 
 	    if ((shell = getenv("SHELL")) == NULL || *shell == '\0')
-		shell = "/bin/sh";
+		shell = DEFAULT_SHELL;
 
 	    argv0 = my_basename(shell);
 	    if (Options & Opt_loginShell) {
@@ -1171,7 +1246,7 @@ init_command(char *argv[])
 	}
 #endif
     get_ourmods();
-    if (Options & Opt_scrollTtyOutputInh)
+    if (!(Options & Opt_scrollTtyOutput))
 	PrivateModes |= PrivMode_TtyOutputInh;
     if (Options & Opt_scrollKeypress)
 	PrivateModes |= PrivMode_Keypress;
@@ -1185,6 +1260,10 @@ init_command(char *argv[])
 #endif
 
     Xfd = XConnectionNumber(Xdisplay);
+
+/*fprintf( stderr, "%s:%d display = %p, Xfd = %d\n\n", __FUNCTION__, __LINE__, Xdisplay, Xfd ); */
+/*	sleep(10); */
+
     cmdbuf_ptr = cmdbuf_endp = cmdbuf_base;
 
     if ((cmd_fd = run_command(argv)) < 0) {
@@ -1213,7 +1292,7 @@ init_xlocale(void)
     Input_Context = NULL;
 
 # ifndef NO_SETLOCALE
-   /* setlocale(LC_CTYPE, "");	/* XXX: should we do this? */
+   /* setlocale(LC_CTYPE, ""); */	/* XXX: should we do this? */
 # endif
     if (rs_inputMethod == NULL || !*rs_inputMethod) {
 	if ((p = XSetLocaleModifiers("")) != NULL && *p)
@@ -1363,7 +1442,7 @@ lookup_key(XEvent * ev)
     {NULL, 0};
     static unsigned char kbuf[KBUFSZ];
     int             ctrl, meta, shft, len;
-    KeySym          keysym;
+    KeySym          keysym = 0;
 
 /*
  * use Num_Lock to toggle Keypad on/off.  If Num_Lock is off, allow an
@@ -1946,11 +2025,14 @@ cmd_getc(void)
     fd_set          readfds;
     int             retval;
 
+	struct timeval  value;
+#if 0
 #ifdef __CYGWIN32__
     struct timeval  value;
 #else
     struct itimerval value;
 #endif
+#endif
 
 /* If there have been a lot of new lines, then update the screen
  * What the heck I'll cheat and only refresh less than every page-full.
@@ -1960,24 +2042,27 @@ cmd_getc(void)
  * refreshing should be correct for small scrolls, because of the
  * time-out */
     if (refresh_count >= (refresh_limit * (TermWin.nrow - 1))) {
-	if (refresh_limit < REFRESH_PERIOD)
-	    refresh_limit++;
-	refresh_count = 0;
-	refreshed = 1;
-	scr_refresh(refresh_type);
+		if (refresh_limit < REFRESH_PERIOD)
+	    	refresh_limit++;
+		refresh_count = 0;
+		refreshed = 1;
+		scr_refresh(refresh_type);
     }
 /* characters already read in */
     if (cmdbuf_ptr < cmdbuf_endp)
 	goto Return_Char;
 
     for (;;) {
+		struct timeval *timeout = &value;
+
 	if (v_bufstr < v_bufptr)	/* output any pending chars */
 	    tt_write(NULL, 0);
 	while (XPending(Xdisplay)) {	/* process pending X events */
 	    XEvent          ev;
 
 	    refreshed = 0;
-	    XNextEvent(Xdisplay, &ev);
+		XNextEvent(Xdisplay, &ev);
+/*fprintf( stderr, "%s:%d Received event %d\n", __FUNCTION__, __LINE__, ev.type );*/
 	    process_x_event(&ev);
 
 	/* in case button actions pushed chars to cmdbuf */
@@ -2004,54 +2089,89 @@ cmd_getc(void)
 	FD_ZERO(&readfds);
 	FD_SET(cmd_fd, &readfds);
 	FD_SET(Xfd, &readfds);
-#ifdef __CYGWIN32__
-	value.tv_usec = TIMEOUT_USEC;
-	value.tv_sec = 0;
-#else
-	value.it_value.tv_usec = TIMEOUT_USEC;
-	value.it_value.tv_sec = 0;
-#endif
 
-	retval = select(num_fds, &readfds, NULL, NULL,
-			((refreshed
+	if( refreshed && last_update_background_request_sec == 0
 #ifndef NO_SCROLLBAR_BUTTON_CONTINUAL_SCROLLING
 			  && !(scrollbar_isUpDn())
 #endif
-#ifdef __CYGWIN32__
-			 ) ? NULL : &value));
-#else
-			 ) ? NULL : &value.it_value));
-#endif
-
+		)
+			timeout = NULL ;
+	else if( !refreshed )
+	{
+		timeout->tv_usec = TIMEOUT_USEC;
+		timeout->tv_sec = 0;
+	}else if( last_update_background_request_sec ) 
+	{
+		time_t curr_t_sec, curr_t_usec, wait_usec = 0 ;	  
+		timer_get_time (&curr_t_sec, &curr_t_usec);
+		if( curr_t_sec == last_update_background_request_sec ) 
+		{
+			if( curr_t_usec < last_update_background_request_usec ) 
+				wait_usec = last_update_background_request_usec - curr_t_usec ; 
+		}else if( curr_t_sec < last_update_background_request_sec ) 
+		{
+			wait_usec = (last_update_background_request_sec - curr_t_sec)*1000000 ;	
+			wait_usec += last_update_background_request_usec ;
+			wait_usec -= curr_t_usec ;
+		}	 
+		if( wait_usec == 0 ) 
+			wait_usec = TIMEOUT_USEC ;
+		timeout->tv_usec = wait_usec;
+		timeout->tv_sec = 0 ; /*UPDATE_BACKGROUND_TIMEOUT_SEC;*/
+	}	 
+/*fprintf( stderr, "%s:%d select(%d, cmd_fd = %d, Xfd = %d, %p)...", __FUNCTION__, __LINE__, num_fds, cmd_fd,Xfd ,timeout ); */
+	retval = select(max(cmd_fd,Xfd)+1, &readfds, NULL, NULL, timeout );
+/*	fprintf( stderr, "Done(retval = %d).\n", retval); */
     /* See if we can read from the application */
 	if (FD_ISSET(cmd_fd, &readfds)) {
-	    unsigned int    count = BUFSIZ;
+	    int             n;
+	    unsigned int    count;
 
 	    cmdbuf_ptr = cmdbuf_endp = cmdbuf_base;
-
-	/* while (count > sizeof(cmdbuf_base) / 2) */
-	    while (count) {
-		int             n = read(cmd_fd, cmdbuf_endp, count);
-
-		if (n <= 0)
+	    for (count = BUFSIZ; count; count -= n, cmdbuf_endp += n)
+		if ((n = read(cmd_fd, cmdbuf_endp, count)) > 0)
+		    continue;
+		else if (n == 0 || (n < 0 && errno == EAGAIN))
 		    break;
-		cmdbuf_endp += n;
-		count -= n;
-	    }
+		else {
+#if !defined(HAVE_ATEXIT) && !defined(__sun__)
+		    clean_exit();
+#endif
+		    exit(EXIT_SUCCESS);
+		}
 	/* some characters read in */
 	    if (count != BUFSIZ)
 		goto Return_Char;
 	}
     /* select statement timed out - better update the screen */
-	if (retval == 0) {
-	    refresh_count = 0;
-	    refresh_limit = 1;
-	    if (!refreshed) {
-		refreshed = 1;
-		scr_refresh(refresh_type);
-		scrollbar_show(1);
-	    }
-	}
+	if (retval == 0) 
+	{
+	    	refresh_count = 0;
+	    	refresh_limit = 1;
+			if( last_update_background_request_sec > 0 )
+			{
+				time_t curr_t_sec, curr_t_usec;	  
+				timer_get_time (&curr_t_sec, &curr_t_usec);
+				if( (	last_update_background_request_sec == curr_t_sec && 
+						last_update_background_request_usec < curr_t_usec) ||
+					last_update_background_request_sec < curr_t_sec || 
+					last_update_background_request_sec-1 > curr_t_sec )
+				{
+					/* TODO update background pixmap */						
+					RenderPixmap(1);
+					refresh_transparent_scrollbar();
+	        		scr_clear();
+					scr_touch();
+					refreshed = True ;
+				}
+			}
+	    	if (!refreshed) 
+			{
+				refreshed = 1;
+				scr_refresh(refresh_type);
+				scrollbar_show(1);
+		    }
+		}
     }
     /* NOTREACHED */
     return 0;
@@ -2116,6 +2236,11 @@ mouse_report(XButtonEvent * ev)
 		      + (MEvent.clicks > 1 ? 8 : 0)
 #endif
 		     ) << 2);
+	/* Report mouse wheel events. */
+	if (ev->button == Button4 || ev->button == Button5) {
+	    key_state |= 1 << 6;
+	    button_number = ev->button - Button4;
+	}
     }
 
 #ifdef DEBUG_MOUSEREPORT
@@ -2149,15 +2274,40 @@ process_x_event(XEvent * ev)
     int             reportmode;
     static int      csrO = 0;	/* Hops - csr offset in thumb/slider      */
 
+
+#if !defined(NO_DEBUG_OUTPUT)
+   	fprintf(stderr, "****************************************************************\n");
+   	fprintf(stderr, "%s:%s:%d(%d)><<EVENT type(%d(%s))->x.window(%lx)->send_event(%d)\n", __FILE__, __FUNCTION__, __LINE__, time(NULL), ev->type, event_type2name(ev->type), ev->xany.window, ev->xany.send_event);
+#endif
+
 /*        to give proper Scroll behaviour */
     switch (ev->type) {
     case KeyPress:
 	lookup_key(ev);
 	break;
 
+	case DestroyNotify :
+		if( ev->xdestroywindow.window == TermWin.vt || ev->xdestroywindow.window == TermWin.parent )
+		{
+#ifdef __CYGWIN__
+    	/* cygwin does not kill shell properly */
+		/* fprintf( stderr, "cmd_pid = %d\n", cmd_pid ); */
+			kill( cmd_pid, SIGKILL );
+#endif	  
+		    exit(EXIT_SUCCESS);
+		}	 
+	    break ;
+
     case ClientMessage:
 	if (ev->xclient.format == 32 && ev->xclient.data.l[0] == wmDeleteWindow)
+	{
+#ifdef __CYGWIN__
+    	/* cygwin does not kill shell properly */
+		/* fprintf( stderr, "cmd_pid = %d\n", cmd_pid ); */
+		kill( cmd_pid, SIGKILL );
+#endif	  
 	    exit(EXIT_SUCCESS);
+	}
 #ifdef OFFIX_DND
     /* OffiX Dnd (drag 'n' drop) protocol */
 	if (ev->xclient.message_type == DndProtocol &&
@@ -2250,6 +2400,7 @@ process_x_event(XEvent * ev)
 	break;
 
     case ConfigureNotify:
+	while( XCheckTypedWindowEvent( Xdisplay, ev->xconfigure.window, ConfigureNotify, ev ) );
 	resize_window(ev);
 	menubar_expose();
 	break;
@@ -2267,75 +2418,121 @@ process_x_event(XEvent * ev)
 	break;
 
      case PropertyNotify:
-#ifdef _MYSTYLE_
-	if (ev->xproperty.atom == _AS_STYLE )
-	{
-	  const char *mystyle_name ;
-	  MyStyle *mystyle ;
+#ifdef DEBUG_BACKGROUND_PMAP    	
+		{
+			char *prop_name = XGetAtomName( Xdisplay, ev->xproperty.atom );
+        	fprintf( stderr, "PropertyNotify : %s(%lX)\n", prop_name, ev->xproperty.atom );
+			if( prop_name )
+	    		XFree( prop_name );
+    	}
+#endif	 	
+		if( ev->xproperty.window == Xroot ) 
+		{	
+#ifdef HAVE_AFTERSTEP
+			if (ev->xproperty.atom == _AS_STYLE ) 
+			{
+				const char *mystyle_name ;
+	    		if( rs_mystyle )
+					mystyle_name = rs_mystyle ;
+	    		else
+					mystyle_name = GetDefaultMyStyle() ;
 
-	    if( rs_mystyle )
-		mystyle_name = rs_mystyle ;
-	    else
-		mystyle_name = GetDefaultMyStyle() ;
-
-	    mystyle_get_property(Xdisplay, Xroot, _AS_STYLE, XA_INTEGER );
-
-	    set_mystyle( mystyle_find( mystyle_name ), True );
-	    break;
-	}
-	else if( TermWin.background.trgType == BGT_MyStyle &&
-		 TermWin.background.mystyle->texture_type < TEXTURE_TRANSPARENT)
-	    break ;
+				mystyle_handle_property_event( ev );
+	    		set_mystyle( mystyle_find( mystyle_name ), True );
+	    		break;
+			}else if( ev->xproperty.atom  == _XROOTPMAP_ID )
+			{	
+				if( TermWin.background.trgType == BGT_MyStyle )
+				{	
+					if(	!TransparentMS(TermWin.background.mystyle) )
+					{
+#ifdef DEBUG_BACKGROUND_PMAP			
+						fprintf( stderr, "texture type = %d\n", TermWin.background.mystyle->texture_type );
+#endif
+	    				break ;
+					}
+					destroy_asimage( &Scr.RootImage );
+				}
+			}
 #endif
 #ifdef TRANSPARENT
-        refresh_transparent_scrollbar();
-        if( Options & Opt_transparent )
-	{
-	    /* if user used some Esetroot compatible prog to set the root
-	     * bg, use the property to determine that. We don't use it's
-	     * value, though */
-
-	    static Atom atom = None ;
-	    if( atom == None ) atom = XInternAtom(Xdisplay, "_XROOTPMAP_ID", True);
-
-	    if( !atom )
-	    {
-		SetSrcPixmap(None);
-	    }else
-	        if (ev->xproperty.atom == atom)
-		{
-		  static Pixmap LastRPixmap = None;
-		  Pixmap currentRPixmap = GetRootPixmap(atom);
-
-	    	    if (currentRPixmap != None )
-		    {
-			if( LastRPixmap != currentRPixmap && TermWin.bMapped && GetMyPosition(NULL, NULL) )
+			if( ev->xproperty.atom  == _XROOTPMAP_ID )
 			{
-/*	fprintf(stderr, "\n Root pixmap has changed");		    */
-    		    	    if( TermWin.background.trgType != BGT_None )
-		    	    {
-		        	SetSrcPixmap(currentRPixmap);
-		        	RenderPixmap(1);
-				/* don't want to check if pixmap is valid
-				   it's unlikely that user would set it to
-				   something bogus. */
-		    	    }
-		    	    scr_clear();
-		    	    scr_touch();
+				if( IsTransparentPixmap() ) 	   
+				{	
+					Pixmap p;
+					if( read_32bit_property (Xroot, _XROOTPMAP_ID, &p) )
+					{	
+						if( p != TermWin.background.srcPixmap ) 
+							SetSrcPixmap( p );
+					}else 
+						ValidateSrcPixmap(True);	  
+				
+#ifdef DEBUG_BACKGROUND_PMAP			   
+					fprintf( stderr, "root pmap changed to = %lX\n", TermWin.background.srcPixmap );
+#endif
+
+					if( TransparentPixmapNeedsUpdate() ) 
+						request_background_update();
+				}else if( get_flags(Options, Opt_transparent) )
+				{
+			        scr_clear();
+	    		    scr_touch();
+				}	 
 			}
-		    }
-		    else
-		    {
-	            	SetSrcPixmap(None);
-/*			fprintf( stderr, "\n aterm's root background set to None");*/
-		    }
-		    /* we want to preserve old pixmap ID so we can check on it
-		       when window is mapped back */
-		    if( TermWin.bMapped ) LastRPixmap = currentRPixmap ;
+#endif
+			if( ev->xproperty.atom  == _XA_NET_SUPPORTING_WM_CHECK )
+			{	
+				check_extended_wm_hints_support();
+#ifdef DEBUG_BACKGROUND_PMAP			   
+				fprintf( stderr, "WM change. Desktops are %ssupported\n", get_flags( ExtWM.flags, WM_SupportsDesktops )?"":"not " );
+#endif
+			}else if( ev->xproperty.atom  == _XA_NET_CURRENT_DESKTOP )
+			{
+				if( !read_32bit_property (Xroot, _XA_NET_CURRENT_DESKTOP, &ExtWM.current_desktop) )
+					clear_flags( ExtWM.flags, WM_SupportsDesktops );
+				else if( get_flags( ExtWM.flags, WM_ClaimSupportsDesktops ) )
+					set_flags( ExtWM.flags, WM_SupportsDesktops	);
+
+				
+#ifdef DEBUG_BACKGROUND_PMAP			   
+				fprintf( stderr, "Curr Desk change to %ld. Desktops are %ssupported\n", ExtWM.current_desktop, get_flags( ExtWM.flags, WM_SupportsDesktops )?"":"not " );
+#endif
+				/* Don't do it here - wait for background change : 
+				 * if( TransparentPixmapNeedsUpdate() ) 
+					request_background_update();
+				 */
+			}	 
 		}
-    	} /* case */
-#endif /* TRANSPARENT */
-	break;
+		if( ev->xproperty.window == TermWin.parent ) 
+		{
+			if( ev->xproperty.atom  == _XA_NET_WM_DESKTOP )
+			{
+#ifdef DEBUG_BACKGROUND_PMAP			   
+				fprintf( stderr, "Aterm Desk change from %ld. Desktops are %ssupported\n", ExtWM.aterm_desktop, get_flags( ExtWM.flags, WM_SupportsDesktops )?"":"not " );
+#endif
+				if( !read_32bit_property (TermWin.parent, _XA_NET_WM_DESKTOP, &ExtWM.aterm_desktop) )
+					clear_flags( ExtWM.flags, WM_SupportsDesktops );
+#ifdef DEBUG_BACKGROUND_PMAP			   
+				fprintf( stderr, "Aterm Desk change to %ld. Desktops are %ssupported\n", ExtWM.aterm_desktop, get_flags( ExtWM.flags, WM_SupportsDesktops )?"":"not " );
+#endif
+				if( TransparentPixmapNeedsUpdate() ) 
+					request_background_update();
+			}else if( ev->xproperty.atom  == _XA_NET_WM_STATE )
+			{
+				if( check_extended_wm_state() ) 
+					if( TransparentPixmapNeedsUpdate() ) 
+						request_background_update();
+#ifdef DEBUG_BACKGROUND_PMAP			   
+				fprintf( stderr, "Aterm state change to: %sSticky, %sShaded, %sHidden\n", 
+								 get_flags( ExtWM.flags, WM_AtermStateSticky )?"":"not ",
+								 get_flags( ExtWM.flags, WM_AtermStateShaded )?"":"not ",
+								 get_flags( ExtWM.flags, WM_AtermStateHidden )?"":"not " );
+#endif
+			
+			}	 
+		}	 
+		break;
      case UnmapNotify:
         TermWin.bMapped = 0 ;
 /*	fprintf(stderr, "\n aterm is UnMapped(event)");*/
@@ -2353,6 +2550,9 @@ process_x_event(XEvent * ev)
 /*        fprintf(stderr, "\n aterm is %s", (TermWin.bMapped)?"Mapped":"UnMapped");*/
 	TermWin.bMapped = 1 ;
 #ifdef TRANSPARENT
+	request_background_update();
+#endif
+#if 0
 	refresh_transparent_scrollbar();
 	if( Options & Opt_transparent)
 	{
@@ -2411,6 +2611,10 @@ process_x_event(XEvent * ev)
     case GraphicsExpose:
     case Expose:
 	if (ev->xany.window == TermWin.vt) {
+#if !defined(NO_DEBUG_OUTPUT)
+   	fprintf(stderr, "exposed area = %dx%d%+d%+d\n", ev->xexpose.x, ev->xexpose.y,
+		       										ev->xexpose.width, ev->xexpose.height);
+#endif
 	    scr_expose(ev->xexpose.x, ev->xexpose.y,
 		       ev->xexpose.width, ev->xexpose.height);
 	} else {
@@ -2623,6 +2827,9 @@ process_x_event(XEvent * ev)
 		Gr_ButtonRelease(ev->xbutton.x, ev->xbutton.y);
 	    else {
 		if (reportmode) {
+		/* Don't report release events for the mouse wheel */
+		if (ev->xbutton.button == Button4 || ev->xbutton.button == Button5)
+		    return;
 		/* mouse report from vt window */
 #ifdef MOUSE_REPORT_DOUBLECLICK
 		/* only report the release of 'slow' single clicks */
@@ -2653,7 +2860,7 @@ process_x_event(XEvent * ev)
 		switch (ev->xbutton.button) {
 		case Button1:
 		case Button3:
-		    selection_make(ev->xbutton.time);
+		    selection_make(ev->xbutton.time, ev->xbutton.state);
 		    break;
 
 		case Button2:
@@ -3280,8 +3487,9 @@ process_terminal_mode(int mode, int priv, unsigned int nargs, int arg[])
 
 	    case 67:
 #ifndef NO_BACKSPACE_KEY
-		if (PrivateModes & PrivMode_HaveBackSpace)
+		if (PrivateModes & PrivMode_HaveBackSpace) {
 		    PrivCases(PrivMode_BackSpace);
+		}
 #endif
 		break;
 
@@ -3298,9 +3506,9 @@ process_terminal_mode(int mode, int priv, unsigned int nargs, int arg[])
 	    case 1010:		/* scroll to bottom on TTY output inhibit */
 		PrivCases(PrivMode_TtyOutputInh);
 		if (PrivateModes & PrivMode_TtyOutputInh)
-		    Options |= Opt_scrollTtyOutputInh;
+		    Options &= ~Opt_scrollTtyOutput;
 		else
-		    Options &= ~Opt_scrollTtyOutputInh;
+		    Options |= Opt_scrollTtyOutput;
 		break;
 	    case 1011:		/* scroll to bottom on key press */
 		PrivCases(PrivMode_Keypress);
@@ -3454,9 +3662,9 @@ void
 main_loop(void)
 {
     int             ch;
-
     do {
 	while ((ch = cmd_getc()) == 0) ;	/* wait for something */
+	/* fprintf( stderr, "%d: cmd_getc returned 0x%x(%c)\n", ++i, ch, isprint(ch)?ch:' ' ); */
 	if (ch >= ' ' || ch == '\t' || ch == '\n' || ch == '\r') {
 	/* Read a text string from the input buffer */
 	    int             nlines = 0;
@@ -3467,20 +3675,34 @@ main_loop(void)
 	 * decrement first since it was post incremented in cmd_getc()
 	 */
 	    str = --cmdbuf_ptr;
-	    while (cmdbuf_ptr < cmdbuf_endp) {
-		ch = *cmdbuf_ptr++;
-		if (ch >= ' ' || ch == '\t' || ch == '\r') {
-		/* nothing */
-		} else if (ch == '\n') {
-		    nlines++;
-		    if (++refresh_count >= (refresh_limit * (TermWin.nrow - 1)))
-			break;
-		} else {	/* unprintable */
-		    cmdbuf_ptr--;
-		    break;
-		}
+	    while (cmdbuf_ptr < cmdbuf_endp) 
+		{
+			ch = *cmdbuf_ptr++;
+			/* fprintf( stderr, "%d: cmd_getc returned 0x%x(%c)\n", ++i, ch, isprint(ch)?ch:' ' ); */
+			if (ch >= ' ' || ch == '\t' ) 
+			{
+			/* nothing */
+			} else if (ch == '\r') 
+			{                  /* we better flush that stuff to create an 
+								* impression that there is something going on */
+				if( cmdbuf_ptr >= cmdbuf_endp || 
+					(*cmdbuf_ptr != '\n'  && *cmdbuf_ptr != '\r')) 
+					break;
+			} else if (ch == '\n') 
+			{
+		    	nlines++;
+		    	if (++refresh_count >= (refresh_limit * (TermWin.nrow - 1)))
+				break;
+			} else {	/* unprintable */
+		    	cmdbuf_ptr--;
+		    	break;
+			}
 	    }
 	    scr_add_lines(str, nlines, (cmdbuf_ptr - str));
+		if( ch == '\r' ) 
+		{	
+			scr_refresh(refresh_type);
+		}
 	} else {
 	    switch (ch) {
 	    case 005:		/* terminal Status */
