@@ -1,10 +1,12 @@
 /*
- * $Id: rxvt.h,v 1.3 2001/03/26 17:26:10 sashav Exp $
+ * $Id: rxvt.h,v 1.16 2005/06/20 16:10:19 sasha Exp $
  */
 
 #ifndef _RXVT_H			/* include once only */
 #define _RXVT_H
 
+#define NEXT_SCROLLBAR
+
 #include "version.h"
 #include "../config.h"
 #include "feature.h"
@@ -140,8 +142,45 @@
 #include <X11/Intrinsic.h>	/* Xlib, Xutil, Xresource, Xfuncproto */
 #include <X11/cursorfont.h>
 #include <X11/keysym.h>
+#include <X11/Xmd.h>
 
-#include "libafterstep.h"
+
+#ifdef HAVE_AFTERIMAGE
+#include <libAfterImage/afterimage.h>
+extern ASVisual *asv;
+#define PREFER_24BIT
+#endif
+
+/* this is needed at all times so that .pro could work */
+struct MyStyle;
+
+#ifdef HAVE_AFTERSTEP
+#include <libAfterStep/afterstep.h>
+#include <libAfterStep/asapp.h>
+#include <libAfterStep/screen.h>
+#include <libAfterStep/mystyle.h>
+#include <libAfterStep/mystyle_property.h>
+#include <libAfterStep/clientprops.h>
+#include <libAfterStep/wmprops.h>
+#else 
+extern Atom  			_XROOTPMAP_ID;
+extern Atom 			_XA_MwmAtom;
+extern Atom 			_XA_NET_WM_PID;
+extern Atom  	_XA_NET_SUPPORTING_WM_CHECK ;
+extern Atom  	_XA_NET_SUPPORTED           ;
+extern Atom  	_XA_NET_CURRENT_DESKTOP     ;
+extern Atom 	_XA_NET_WM_DESKTOP;
+extern Atom 	_XA_NET_WM_STATE;
+extern Atom 	_XA_NET_WM_STATE_STICKY;
+extern Atom 	_XA_NET_WM_STATE_SHADED;
+extern Atom 	_XA_NET_WM_STATE_HIDDEN;
+#ifndef get_flags
+#define get_flags(var, val) 	(((var) & (val)))  /* making it sign safe */
+#define set_flags(var, val) 	((var) |= (val))
+#define clear_flags(var, val) 	((var) &= ~(val))
+#endif
+
+#endif
 
 #ifdef DEBUG_X
 #define XGetGeometry(dpy,win,r,x,y,w,h,b,d) \
@@ -173,6 +212,7 @@ typedef RUINT32T	R_u_int32_t;
 typedef RINTPT		R_int_p_t;
 typedef RUINTPT		R_u_int_p_t;
 
+
 /*
  *****************************************************************************
  * STRUCTURES AND TYPEDEFS
@@ -197,12 +237,16 @@ typedef RUINTPT		R_u_int_p_t;
 
 #if defined(BACKGROUND_IMAGE) || defined(TRANSPARENT) || defined(_MYSTYLE_)
 
+#ifndef HAVE_AFTERIMAGE
+
 typedef struct ShadingInfo{
     XColor      tintColor ;
     int shading ;
 }ShadingInfo ;
-
 #define NO_NEED_TO_SHADE(o) (o.shading==100 && o.tintColor.red==0xFFFF && o.tintColor.green==0xFFFF && o.tintColor.blue == 0xFFFF)
+
+#endif
+
 #define INIT_SHADING(o)     {o.shading=100; o.tintColor.red=0xFFFF; o.tintColor.green=0xFFFF; o.tintColor.blue = 0xFFFF;}
 struct MyStyle;
 
@@ -256,9 +300,13 @@ typedef struct _BackroundInfo {
 }BackgroundInfo ;
 #endif
 
+#ifndef NO_DEBUG_OUTPUT
+#define DEBUG_BACKGROUND_PMAP
+#endif
+
 
 typedef struct {
-    R_int16_t       width,	/* window width [pixels]                    */
+    R_u_int16_t     width,	/* window width [pixels]                    */
 		    height,	/* window height [pixels]                   */
 		    fwidth,	/* font width [pixels]                      */
 		    fheight,	/* font height [pixels]                     */
@@ -283,6 +331,8 @@ typedef struct {
 #if defined(BACKGROUND_IMAGE) || defined(TRANSPARENT) || defined(_MYSTYLE_)
     BackgroundInfo background;
 #endif
+	/* geometry of the parent window at the time of the last ConfigureNotify */
+	int root_x, root_y, root_width, root_height ; 
 
 #define GC_TYPE_AND 	"and"
 #define GC_TYPE_AND_REV	"andReverse"
@@ -302,7 +352,8 @@ typedef struct {
 
 #if defined(BACKGROUND_IMAGE) || defined(TRANSPARENT) || defined(_MYSTYLE_)
     GC              tintGC;
-    Pixmap	    LastPixmapUsed ;
+    Pixmap	    	LastPixmapUsed ;
+	int 			LastPixmap_width, LastPixmap_height, LastPixmap_root_x, LastPixmap_root_y ; 
 #endif
 
     int		    bMapped;
@@ -458,6 +509,45 @@ typedef struct {
 	(tmp) = (one); (one) = (two); (two) = (tmp);	\
     } while (0)
 
+#ifndef CLIENTPROPS_H_HEADER_INCLUDED
+
+/* Motif window hints */
+#define MWM_HINTS_FUNCTIONS     (1L << 0)
+#define MWM_HINTS_DECORATIONS   (1L << 1)
+#define MWM_HINTS_INPUT_MODE    (1L << 2)
+#define MWM_HINTS_STATUS        (1L << 3)
+/* bit definitions for MwmHints.functions */
+#define MWM_FUNC_ALL            (1L << 0)
+#define MWM_FUNC_RESIZE         (1L << 1)
+#define MWM_FUNC_MOVE           (1L << 2)
+#define MWM_FUNC_MINIMIZE       (1L << 3)
+#define MWM_FUNC_MAXIMIZE       (1L << 4)
+#define MWM_FUNC_CLOSE          (1L << 5)
+/* bit definitions for MwmHints.decorations */
+#define MWM_DECOR_ALL           (1L << 0)
+#define MWM_DECOR_BORDER        (1L << 1)
+#define MWM_DECOR_RESIZEH       (1L << 2)
+#define MWM_DECOR_TITLE         (1L << 3)
+#define MWM_DECOR_MENU          (1L << 4)
+#define MWM_DECOR_MINIMIZE      (1L << 5)
+#define MWM_DECOR_MAXIMIZE      (1L << 6)
+/* bit definitions for MwmHints.inputMode */
+#define MWM_INPUT_MODELESS                  0
+#define MWM_INPUT_PRIMARY_APPLICATION_MODAL 1
+#define MWM_INPUT_SYSTEM_MODAL              2
+#define MWM_INPUT_FULL_APPLICATION_MODAL    3
+#define PROP_MWM_HINTS_ELEMENTS             5
+
+
+typedef struct MwmHints {
+  CARD32 flags;
+  CARD32 functions;
+  CARD32 decorations;
+  INT32  input_mode;
+  CARD32 status;
+} MwmHints;
+
+#endif
 /*
  *****************************************************************************
  * NORMAL DEFINES
@@ -468,6 +558,8 @@ typedef struct {
 # define OLD_WORD_SELECTION
 #endif
 
+#define DEFAULT_SHELL "/bin/bash"
+
 /* sort out conflicts in feature.h */
 #ifdef KANJI
 # undef ZH			/* remove Chinese big5 support        */
@@ -593,10 +685,6 @@ typedef struct {
 #define scrollBar_esc		30
 #define menuBar_margin		2	/* margin below text */
 
-/* gap between text and window edges (could be configurable) */
-#define TermWin_internalBorder	2
-#define TermWin_internalBorders	4 /* must be two times of value above */
-
 /* width of scrollBar, menuBar shadow, must be 1 or 2 */
 #ifdef HALFSHADOW
 # define SHADOW 1
@@ -686,12 +774,14 @@ enum {
 #define Opt_scrollBar_right	(1LU<<8)
 #define Opt_scrollBar_floating	(1LU<<9)
 #define Opt_meta8		(1LU<<10)
-#define Opt_scrollTtyOutputInh	(1LU<<11)
+#define Opt_scrollTtyOutput	(1LU<<11)
 #define Opt_scrollKeypress	(1LU<<12)
 #define Opt_transparent		(1LU<<13)
 #define Opt_transparent_sb	(1LU<<14)
+#define Opt_borderLess		(1LU<<16)
 
 /* place holder used for parsing command-line options */
+#define Opt_Reverse		(1LU<<30)
 #define Opt_Boolean		(1LU<<31)
 
 /*
@@ -700,6 +790,11 @@ enum {
 #define XTerm_name		0
 #define XTerm_iconName		1
 #define XTerm_title		2
+#define XTerm_Color		4	/* change colors */
+#define XTerm_Color_cursor	12	/* change actual 'Cursor' color */
+#define XTerm_Color_pointer	13	/* change actual 'Pointer' color */
+#define XTerm_Color_BD		18	/* change actual 'Bold' color */
+#define XTerm_Color_UL		19	/* change actual 'Underline' color */
 #define XTerm_logfile		46	/* not implemented */
 #define XTerm_font		50
 
@@ -779,6 +874,7 @@ enum colour_list {
  */
 #define	NGRX_PTS	1000
 
+
 /*
  *****************************************************************************
  * MACRO DEFINES
@@ -885,19 +981,9 @@ enum colour_list {
 # define EXTERN extern
 #endif
 
-#ifdef PREFER_24BIT
 EXTERN Colormap     Xcmap;
 EXTERN int          Xdepth;
 EXTERN Visual      *Xvisual;
-#else
-# define Xcmap			DefaultColormap(Xdisplay,Xscreen)
-# define Xdepth			DefaultDepth(Xdisplay,Xscreen)
-# define Xvisual		DefaultVisual(Xdisplay,Xscreen)
-# ifdef DEBUG_DEPTH
-#  undef Xdepth
-#  define Xdepth		DEBUG_DEPTH
-# endif
-#endif
 
 EXTERN TermWin_t	TermWin;
 #define PARENTS_NUM 3
@@ -905,9 +991,36 @@ EXTERN TermWin_t	TermWin;
 extern Window		ParentWin[PARENTS_NUM];
 extern int		ParentWinNum;
 
+/* gap between text and window edges (could be configurable) */
+EXTERN int TermWin_internalBorder;
+/* must be two times of value above */
+EXTERN int TermWin_internalBorders;
+
+
 EXTERN scrollBar_t	scrollBar;
 EXTERN menuBar_t	menuBar;
 EXTERN Display	       *Xdisplay;
+EXTERN int	       		XdisplayWidth, XdisplayHeight;
+
+typedef struct ExtWMSupportData
+{
+#define WM_SupportsDesktops 		(0x01<<0)	
+#define WM_AtermStateSticky			(0x01<<1)	  
+#define WM_AtermStateShaded 		(0x01<<2)	  
+#define WM_AtermStateHidden 		(0x01<<3)	  
+#define WM_ClaimSupportsDesktops 	(0x01<<4)	  
+
+	unsigned long flags ; 	
+	Window supporting_wm_check ; 
+	CARD32 current_desktop;
+	CARD32 aterm_desktop;
+}ExtWMSupportData;
+
+EXTERN ExtWMSupportData ExtWM; 
+
+EXTERN Bool	wm_supports_desktops ;
+EXTERN int	wm_current_desktop, wm_aterm_desktop ;
+
 EXTERN unsigned long	Options;
 EXTERN int		sb_shadow;
 EXTERN int		delay_menu_drawing;
@@ -929,6 +1042,7 @@ EXTERN const char      *rs_geometry;	/* window geometry */
 EXTERN const char      *rs_minBufferWidth; /* minimum buffer width - so we can scroll horizontally */
 EXTERN const char      *rs_saveLines;	/* scrollback buffer [lines] */
 EXTERN const char      *rs_borderWidth; /* border width [pixels] */
+EXTERN const char      *rs_internal_border; /* internal border width [pixels] */
 EXTERN const char      *rs_cutchars;	/* chars for selection boundaries */
 #ifdef META8_OPTION
 EXTERN const char      *rs_modifier;
@@ -977,6 +1091,14 @@ EXTERN const char      *rs_fade;
 #endif
 
 EXTERN const char      *rs_textType;
+
+EXTERN Atom     		aterm_XA_TARGETS;
+EXTERN Atom     		aterm_XA_TEXT;
+EXTERN Atom     		_XA_COMPAUND_TEXT;
+EXTERN Atom     		aterm_XA_UTF8_STRING;
+EXTERN Atom     		aterm_XA_CLIPBOARD;
+
+
 /*
  *****************************************************************************
  * PROTOTYPES
