@@ -35,7 +35,7 @@
  *----------------------------------------------------------------------*/
 
 #ifndef lint
-static const char rcsid[] = "$Id: xdefaults.c,v 1.3 2001/08/09 05:25:05 sashav Exp $";
+static const char rcsid[] = "$Id: xdefaults.c,v 1.4 2005/06/03 20:11:55 sasha Exp $";
 #endif
 
 #include "rxvt.h"		/* NECESSARY */
@@ -44,12 +44,13 @@ static const char rcsid[] = "$Id: xdefaults.c,v 1.3 2001/08/09 05:25:05 sashav E
 
 /* local functions referenced */
 /*{{{ local variables */
+static const char *rs_borderLess = NULL;
 static const char *rs_loginShell = NULL;
 static const char *rs_utmpInhibit = NULL;
 static const char *rs_scrollBar = NULL;
 static const char *rs_scrollBar_right = NULL;
 static const char *rs_scrollBar_floating = NULL;
-static const char *rs_scrollTtyOutputInh = NULL;
+static const char *rs_scrollTtyOutput = NULL;
 static const char *rs_scrollKeypress = NULL;
 
 #ifdef TRANSPARENT
@@ -57,12 +58,6 @@ static const char *rs_transparent = NULL;
 static const char *rs_transparent_sb = NULL;
 #endif
 
-#ifdef DONT_TILE_PIXMAP_OPTION
-static const char *rs_dontTilePixmap = NULL;
-static const char *rs_dontTilePixmapH = NULL;
-static const char *rs_dontTilePixmapV = NULL;
-#endif
-
 #if defined (HOTKEY_CTRL) || defined (HOTKEY_META)
 static const char *rs_bigfont_key = NULL;
 static const char *rs_smallfont_key = NULL;
@@ -126,6 +121,8 @@ static const struct {
     (optList[i].flag ? 0 : (optList[i].arg ? strlen (optList[i].arg) : 1))
 #define optList_isBool(i)		\
     (optList[i].flag & Opt_Boolean)
+#define optList_isReverse(i)		\
+    (optList[i].flag & Opt_Reverse)
 #define optList_size()			\
     (sizeof(optList) / sizeof(optList[0]))
 
@@ -297,17 +294,22 @@ static const struct {
          "pointer color"),
     STRG(rs_borderWidth, "borderWidth", "bw", "number",
          "width of border"),
+    STRG(rs_internal_border, "internalBorder", "ib", "number",
+         "width of the internal border"),
     STRG(rs_color[Color_border], "borderColor", "bd", "color",
          "border color"),
+    BOOL(rs_borderLess, "borderLess", "bl", Opt_borderLess, "no decoration"),
     BOOL(rs_loginShell, "loginShell", "ls", Opt_loginShell, "login shell"),
     BOOL(rs_scrollBar, "scrollBar", "sb", Opt_scrollBar, "scrollbar"),
     BOOL(rs_scrollBar_right, "scrollBar_right", "sr", Opt_scrollBar_right,
          "scrollbar right"),
     BOOL(rs_scrollBar_floating, "scrollBar_floating", "st",
          Opt_scrollBar_floating, "scrollbar without a trough"),
-    BOOL(rs_scrollTtyOutputInh, "scrollTtyOutput", "si",
-         Opt_scrollTtyOutputInh, "scroll-on-tty-output inhibit"),
-    BOOL(rs_scrollKeypress, "scrollTtyKeypress", "sk",
+    BOOL(rs_scrollTtyOutput, "scrollTtyOutput", NULL,
+         Opt_scrollTtyOutput, NULL),
+    BOOL(rs_scrollTtyOutput, NULL, "si", Opt_Reverse | Opt_scrollTtyOutput,
+	 "scroll-on-tty-output inhibit"),
+    BOOL(rs_scrollKeypress, "scrollKey", "sk",
          Opt_scrollKeypress, "scroll-on-keypress"),
     STRG(rs_minBufferWidth, "minBufferWidth", "mbw", "number",
          "minimum number of columns stored in buffer"),
@@ -593,6 +595,8 @@ get_options(int argc, char *argv[])
 		break;
 
 	if (entry < optList_size()) {
+	    if (optList_isReverse(entry))
+		flag = flag == On ? Off : On;
 	    if (optList_strlen(entry)) {	/* string value */
 		char           *str = argv[++i];
 
@@ -844,12 +848,19 @@ get_xdefaults(FILE * stream, const char *name)
 		    n = (str ? strlen(str) : 0);
 		    if (n && *(optList[entry].dp) == NULL) {
 		    /* not already set */
+			int		s;
 			char           *p = MALLOC((n + 1) * sizeof(char));
 
 			STRCPY(p, str);
 			*(optList[entry].dp) = p;
 			if (optList_isBool(entry)) {
-			    if (!my_strcasecmp(str, "TRUE"))
+			    s = my_strcasecmp(p, "TRUE") == 0
+				|| my_strcasecmp(p, "YES") == 0
+				|| my_strcasecmp(p, "ON") == 0
+				|| my_strcasecmp(p, "1") == 0;
+			    if (optList_isReverse(entry))
+				s = !s;
+			    if (s)
 				Options |= (optList[entry].flag);
 			    else
 				Options &= ~(optList[entry].flag);
@@ -905,18 +916,34 @@ extract_resources(Display * display, const char *name)
 #  endif
 
     for (entry = 0; entry < optList_size(); entry++) {
-	char           *p;
+	int		s;
+	char           *p, *p0;
 	const char     *kw = optList[entry].kw;
 
 	if (kw == NULL || *(optList[entry].dp) != NULL)
 	    continue;		/* previously set */
-	if ((p = XGetDefault(display, name, kw)) != NULL ||
-	    (p = XGetDefault(display, APL_SUBCLASS, kw)) != NULL ||
-	    (p = XGetDefault(display, APL_CLASS, kw)) != NULL) {
+
+#define STRCMP(x, y)		strcmp((const char *)(x), (const char *)(y))
+	p = XGetDefault(display, name, kw);
+	p0 = XGetDefault(display, "!INVALIDPROGRAMMENAMEDONTMATCH!", kw);
+	if (p == NULL || (p0 && STRCMP(p, p0) == 0)) {
+	    p = XGetDefault(display, APL_SUBCLASS, kw);
+	    if (p == NULL || (p0 && STRCMP(p, p0) == 0))
+		p = XGetDefault(display, APL_CLASS, kw);
+	}
+	if (p == NULL && p0)
+	    p = p0;
+	if (p) {
 	    *optList[entry].dp = p;
 
 	    if (optList_isBool(entry)) {
-		if (!my_strcasecmp(p, "TRUE"))
+		s = my_strcasecmp(p, "TRUE") == 0
+		    || my_strcasecmp(p, "YES") == 0
+		    || my_strcasecmp(p, "ON") == 0
+		    || my_strcasecmp(p, "1") == 0;
+		if (optList_isReverse(entry))
+		    s = !s;
+		if (s)
 		    Options |= (optList[entry].flag);
 		else
 		    Options &= ~(optList[entry].flag);
